﻿using System;

namespace Ascon.AppPlugin
{
    class AppProviderWrapper : IAppProvider
    {
        private readonly IPluginAppProvider appProvider;
        private readonly IAppMain appMain;
        private readonly IAppWorkFlow appWorkFlow;
        private readonly IAppCache appCache;
        private readonly IAppConfigurator appConfigurator;

        public AppProviderWrapper(IPluginAppProvider appProvider)
        {
            if (appProvider == null)
                throw new ArgumentNullException("appProvider");
            this.appProvider = appProvider;
            appMain = new AppMainWrapper(appProvider.GetAppMain());
            appWorkFlow = new AppWorkFlowWrapper(appProvider.GetAppWorkFlow());
            appCache = new AppCacheWrapper(appProvider.GetAppCache());
            appConfigurator = new AppConfiguratorWrapper(appProvider.GetAppConfigurator());
        }

        private void ApiCheck(bool res)
        {
            if (!res)
                throw new Exception(appProvider.GetLastError());
        }

        public IAppMain AppMain
        {
            get { return appMain; }
        }

        public IAppCache AppCache
        {
            get { return appCache; }
        }

        public IAppWorkFlow AppWorkFlow
        {
            get { return appWorkFlow; }
        }

        public IAppConfigurator AppConfigurator
        {
            get { return appConfigurator; }
        }

        public string GetAppTitle()
        {
            return appProvider.GetAppTitle();
        }

        public int GetAppHandle()
        {
            return appProvider.GetAppHandle();
        }

        public IPluginPDMVersions GetSelectedObjects()
        {
            return appProvider.GetSelectedObjects();
        }

        public void RefreshVersion(int idVersion)
        {
            appProvider.RefreshVersion(idVersion);
        }

        public void SelectObject(int idVersion)
        {
            ApiCheck(appProvider.SelectObject(idVersion));
        }

        public void SelectTask(int idProcess, TTasksFilterExp TasksFilter)
        {
            ApiCheck(appProvider.SelectTask(idProcess, TasksFilter));
        }

        public void SelectMail(int idLetter, TMailFilterExp Filter)
        {
            ApiCheck(appProvider.SelectMail(idLetter, Filter));
        }

        public IPluginTasks GetSelectedTasks()
        {
            IPluginTasks res;
            ApiCheck(appProvider.GetSelectedTasks(out res));
            return res;
        }

        public TModalResultExp GetAttachFromDlg(out IPluginPDMVersions Versions)
        {
            TModalResultExp res;
            ApiCheck(appProvider.GetAttachFromDlg(out Versions, out res));
            return res;
        }

        public TModalResultExp GetUsersFromDlg(bool ADateVisible, bool ASignRoleVisible, bool AMultiselect, bool ACan_I_Am, IPluginUsers ASelectedUsers, out IPluginUsers User)
        {
            TModalResultExp res;
            ApiCheck(appProvider.GetUsersFromDlg(ADateVisible, ASignRoleVisible, AMultiselect, ACan_I_Am, ASelectedUsers, out User, out res));
            return res;
        }

        public void RefreshTasks(int ASelectedRouteId)
        {
            ApiCheck(appProvider.RefreshTasks(ASelectedRouteId));
        }
    }
}
