﻿using System;

namespace Ascon.AppPlugin
{
	public class AppWorkFlowWrapper : IAppWorkFlow
	{
		private readonly IPluginAppWorkFlow appWorkFlow;

		public AppWorkFlowWrapper(IPluginAppWorkFlow appWorkFlow)
		{
			if (appWorkFlow == null)
				throw new ArgumentNullException("appWorkFlow");

			this.appWorkFlow = appWorkFlow;
		}

		private void ApiCheck(bool res)
		{
			if (!res)
				throw new Exception(appWorkFlow.GetLastError());
		}

		public int CreateTask(string subj, string text, int idVersion, int idRole, int idUser, DateTime datePlan, int criticalTerm, bool checkResult, TBProcessKindExp kind)
		{
			int res;
			ApiCheck(appWorkFlow.CreateTask(subj, text, idVersion, idRole, idUser, datePlan, criticalTerm, checkResult, kind, out res));
			return res;
		}

		public void StartTask(int idRoute, int idRole, int idUser, string subj, DateTime datePlan)
		{
			ApiCheck(appWorkFlow.StartTask(idRoute, idRole, idUser, subj, datePlan));
		}

		public int CreateTaskByDialog()
		{
			int res;
			ApiCheck(appWorkFlow.CreateTaskByDialog(out res));
			return res;
		}

		public IPluginDataSet GetIncomingCorrespondence(int inRetMode)
		{
			IPluginDataSet ds;
			ApiCheck(appWorkFlow.GetIncomingCorrespondence(inRetMode, out ds));
			return ds;
		}

		public IPluginDataSet GetOutcomingCorrespondence(int inRetMode)
		{
			IPluginDataSet ds;
			ApiCheck(appWorkFlow.GetOutcomingCorrespondence(inRetMode, out ds));
			return ds;
		}

		public IPluginDataSet GetCorrespondenceList(int inIdRoute)
		{
			IPluginDataSet ds;
			ApiCheck(appWorkFlow.GetCorrespondenceList(inIdRoute, out ds));
			return ds;
		}

		public void DelLetter(int inIdLetter)
		{
			ApiCheck(appWorkFlow.DelLetter(inIdLetter));
		}

		public IPluginDataSet GetUserList()
		{
			IPluginDataSet ds;
			ApiCheck(appWorkFlow.GetUserList(out ds));
			return ds;
		}

		public IPluginDataSet GetRoleTree(int inMode)
		{
			IPluginDataSet ds;
			ApiCheck(appWorkFlow.GetRoleTree(inMode, out ds));
			return ds;
		}

		public IPluginDataSet GetProcessListByObject(string stDBName, int inPdmId)
		{
			IPluginDataSet ds;
			ApiCheck(appWorkFlow.GetProcessListByObject(stDBName, inPdmId, out ds));
			return ds;
		}

		public IPluginDataSet GetProcessListByObjectEx(int inPdmId, string stDBName)
		{
			IPluginDataSet ds;
			ApiCheck(appWorkFlow.GetProcessListByObjectEx(inPdmId, stDBName, out ds));
			return ds;
		}

		public IPluginDataSet GetInitialProcessList()
		{
			IPluginDataSet ds;
			ApiCheck(appWorkFlow.GetInitialProcessList(out ds));
			return ds;
		}

		public IPluginDataSet GetInfoAboutCurrentUser()
		{
			IPluginDataSet ds;
			ApiCheck(appWorkFlow.GetInfoAboutCurrentUser(out ds));
			return ds;
		}

		public IPluginDataSet GetInfoAboutRoute(int inIdRoute)
		{
			IPluginDataSet ds;
			ApiCheck(appWorkFlow.GetInfoAboutRoute(inIdRoute, out ds));
			return ds;
		}

		public IPluginDataSet GetRoleListForUser(int inIdUser)
		{
			IPluginDataSet ds;
			ApiCheck(appWorkFlow.GetRoleListForUser(inIdUser, out ds));
			return ds;
		}

		public IPluginDataSet GetTaskListForUser(int inIdUser, int inUserRoleMask, int inProcessStateMask, int inStageStateMode)
		{
			IPluginDataSet ds;
			ApiCheck(appWorkFlow.GetTaskListForUser(inIdUser, inUserRoleMask, inProcessStateMask, inStageStateMode, out ds));
			return ds;
		}

		public int CreateNewProcess(string stDBName, int inIdPdmObject, string stRouteName)
		{
			int res;
			ApiCheck(appWorkFlow.CreateNewProcess(stDBName, inIdPdmObject, stRouteName, out res));
			return res;
		}

		public void DeleteProcess(int inIdRoute)
		{
			ApiCheck(appWorkFlow.DeleteProcess(inIdRoute));
		}

		public void InsUpdStageIntoBp(string stTask, int inCriticalTerm, DateTime dtDatePlan, int inCanControl, int inActive,
			int inIdRole, int inIdUser, int inIdBProc, int inCanTransform, ref int inIdStage, ref int inTimeConstraint, ref int inFromBpStart)
		{
			ApiCheck(appWorkFlow.InsUpdStageIntoBP(stTask, inCriticalTerm, dtDatePlan, inCanControl, inActive, inIdRole, inIdUser, inIdBProc, inCanTransform, ref inIdStage, ref inTimeConstraint, ref inFromBpStart));
		}

		public void AddRouteItemsSeq(int inIdFirst, int inIdSecond, int inType)
		{
			ApiCheck(appWorkFlow.AddRouteItemsSeq(inIdFirst, inIdSecond, inType));
		}

		public void AddRouteItemsRetSeq(int inIdFirst, int inIdSecond)
		{
			ApiCheck(appWorkFlow.AddRouteItemsRetSeq(inIdFirst, inIdSecond));
		}

		public IPluginDataSet GetRoleListIntoProcess(int inIdRoute)
		{
			IPluginDataSet ds;
			ApiCheck(appWorkFlow.GetRoleListIntoProcess(inIdRoute, out ds));
			return ds;
		}

		public void SetStateOfProcess(int inIdRoute, int inStateBp)
		{
			ApiCheck(appWorkFlow.SetStateOfProcess(inIdRoute, inStateBp));
		}

		public void InsUpdUser(string stUserName, string stDescr, string stEMail, int inIsAdmin, ref int vaIdUser)
		{
			ApiCheck(appWorkFlow.InsUpdUser(stUserName, stDescr, stEMail, inIsAdmin, ref vaIdUser));
		}

		public void SetTaskState(int inIdRoute, int inIdOperation, int inStateTask, string stMessageBody)
		{
			ApiCheck(appWorkFlow.SetTaskState(inIdRoute, inIdOperation, inStateTask, stMessageBody));
		}

		public void SetItemState(int inIdItem, int inState, int inAddSignature)
		{
			ApiCheck(appWorkFlow.SetItemState(inIdItem, inState, inAddSignature));
		}

		public void SetItemStateInternal(int inIdItem, int inState)
		{
			ApiCheck(appWorkFlow.SetItemStateInternal(inIdItem, inState));
		}

		public void AddBpDocument(int inIdRoute, int inIdDoc, int inIdPar)
		{
			ApiCheck(appWorkFlow.AddBPDocument(inIdRoute, inIdDoc, inIdPar));
		}

		public void DelBpDocument(int inIdRoute, int inIdDoc)
		{
			ApiCheck(appWorkFlow.DelBPDocument(inIdRoute, inIdDoc));
		}

		public IPluginDataSet GetFirstItems(int inIdRoute)
		{
			IPluginDataSet ds;
			ApiCheck(appWorkFlow.GetFirstItems(inIdRoute, out ds));
			return ds;
		}

		public IPluginDataSet GetNextItems(int inIdItem)
		{
			IPluginDataSet ds;
			ApiCheck(appWorkFlow.GetNextItems(inIdItem, out ds));
			return ds;
		}

		public IPluginDataSet rep_ExecutedJobs(int inIdParent)
		{
			IPluginDataSet ds;
			ApiCheck(appWorkFlow.rep_ExecutedJobs(inIdParent, out ds));
			return ds;
		}

		public IPluginDataSet GetJobsListIntoProcess(int inIdRoute)
		{
			IPluginDataSet ds;
			ApiCheck(appWorkFlow.GetJobsListIntoProcess(inIdRoute, out ds));
			return ds;
		}

		public IPluginDataSet GetAttachDocumentListIntoProcess(int inIdRoute)
		{
			IPluginDataSet ds;
			ApiCheck(appWorkFlow.GetAttachDocumentListIntoProcess(inIdRoute, out ds));
			return ds;
		}

		public void StartNextStage(int inIdRoute, int inIdItem, int inDirect)
		{
			ApiCheck(appWorkFlow.StartNextStage(inIdRoute, inIdItem, inDirect));
		}

		public void RenameProcess(int inIdRoute, string stNewName)
		{
			ApiCheck(appWorkFlow.RenameProcess(inIdRoute, stNewName));
		}

		public void SetStageExecutor(int inIdStage, int inIdUser, int inIdRole)
		{
			ApiCheck(appWorkFlow.SetStageExecutor(inIdStage, inIdUser, inIdRole));
		}

		public void SetStageTerm(int inIdStage, int inTimeLimit, DateTime vaPlanEnd, int inTimeConstraint, int inFromBpStart)
		{
			ApiCheck(appWorkFlow.SetStageTerm(inIdStage, inTimeLimit, vaPlanEnd, inTimeConstraint, inFromBpStart));
		}

		public void SetStageTask(int inIdStage, string stTask)
		{
			ApiCheck(appWorkFlow.SetStageTask(inIdStage, stTask));
		}

		public void SetBP_PDMObject(int inIdRoute, int inIdPdmObject)
		{
			ApiCheck(appWorkFlow.SetBP_PDMObject(inIdRoute, inIdPdmObject));
		}

		public IPluginDataSet GetTaskListForObject(int inIdVersion, int inProcessStateMask, int inStageStateMode)
		{
			IPluginDataSet ds;
			ApiCheck(appWorkFlow.GetTaskListForObject(inIdVersion, inProcessStateMask, inStageStateMode, out ds));
			return ds;
		}

		public IPluginDataSet GetInfoAboutStage(int inIdStage)
		{
			IPluginDataSet ds;
			ApiCheck(appWorkFlow.GetInfoAboutStage(inIdStage, out ds));
			return ds;
		}

		public void GetVariable(int inIdRoute, string stName, out int res)
		{
			ApiCheck(appWorkFlow.GetVariable(inIdRoute, stName, out res));
		}

		public void GetVariable(int inIdRoute, string stName, out string res)
		{
			ApiCheck(appWorkFlow.GetVariable(inIdRoute, stName, out res));
		}

		public void GetVariable(int inIdRoute, string stName, out DateTime res)
		{
			ApiCheck(appWorkFlow.GetVariable(inIdRoute, stName, out res));
		}

		public void GetVariable(int inIdRoute, string stName, out double res)
		{
			ApiCheck(appWorkFlow.GetVariable(inIdRoute, stName, out res));
		}

		public void SetVariable(int inIdRoute, string stName, int value, bool readOnlyForChild)
		{
			ApiCheck(appWorkFlow.SetVariable(inIdRoute, stName, value, readOnlyForChild));
		}

		public void SetVariable(int inIdRoute, string stName, string value, bool readOnlyForChild)
		{
			ApiCheck(appWorkFlow.SetVariable(inIdRoute, stName, value, readOnlyForChild));
		}

		public void SetVariable(int inIdRoute, string stName, DateTime value, bool readOnlyForChild)
		{
			ApiCheck(appWorkFlow.SetVariable(inIdRoute, stName, value, readOnlyForChild));
		}

		public void SetVariable(int inIdRoute, string stName, double value, bool readOnlyForChild)
		{
			ApiCheck(appWorkFlow.SetVariable(inIdRoute, stName, value, readOnlyForChild));
		}

		public int SendLetterByBPToOneUser(int inIdRoute, int inIdAURole, int inIdRecRole, int inIdRecUser, string stSubj, string stBody)
		{
			int res;
			ApiCheck(appWorkFlow.SendLetterByBPToOneUser(inIdRoute, inIdAURole, inIdRecRole, inIdRecUser, stSubj, stBody, out res));
			return res;
		}

		public int SendLetterByBPToUsers(int inIdRoute, int inIdAURole, int inIdRecRole, int inIdRecUser, string stSubj, string stBody)
		{
			int result;
			ApiCheck(appWorkFlow.SendLetterByBPToOneUser(inIdRoute, inIdAURole, inIdRecRole, inIdRecUser, stSubj, stBody, out result));
			return result;
		}

		public IPluginDataSet GetTypicalProcessList(int inMode)
		{
			IPluginDataSet res;
			ApiCheck(appWorkFlow.GetTypicalProcessList(inMode, out res));
			return res;
		}

		public int CreateNewProcessBy(string stDBName, int inID_PDM_OBJECT, string stRouteName, int inIdBaseRoute)
		{
			int res;
			ApiCheck(appWorkFlow.CreateNewProcessBy(stDBName, inID_PDM_OBJECT, stRouteName, inIdBaseRoute, out res));
			return res;
		}

		public int InsUpdOpInstance(int inIdOperDescr, int inIsWait, int inIdProc, int inIdOperInst)
		{
			int res;
			ApiCheck(appWorkFlow.InsUpdOpInstance(inIdOperDescr, inIsWait, inIdProc, inIdOperInst, out res));
			return res;
		}
	}
}