﻿unit Plugin;

interface

uses
  Windows,
  SysUtils,

  IniProperties,
  BarCodeCom,
  PluginIntf,
  PluginIntfWrapper;

type
  TPlugin = class(TInterfacedObject, IPlugin, IPluginMenu)
  private
    FLastError: WideString;
    FAppProvider: TAppProvider;
    FBarCode: TBarCode;
    FComPort: string;
    FIni: TIniProperties;
    procedure ShowForm;
    procedure ShowException(const AMessage: string);
  public
    // методы интерфейса IPlugin
    function GetVersion(): Integer; safecall;
    function GetLastError(): WideString; safecall;
    procedure Init(AppProvider: IPluginAppProvider); safecall;
    // IPluginMenu
    procedure InitMainMenu(AMenu: IPluginMainMenu); safecall;
    procedure CallMainMenu(AIndex: Integer); safecall;
    // TPlugin
    constructor Create;
    destructor Destroy; override;
  end;

function LoadPlugin(): IPlugin; safecall;

implementation

uses
  Forms,
  Properties,
  Controls,
  Dialogs;

function LoadPlugin(): IPlugin;
begin
  Result := TPlugin.Create;
end;

{ TPlugin }

constructor TPlugin.Create;
begin
  FIni := TIniProperties.Create;
end;

destructor TPlugin.Destroy;
begin
  // обязательно устанавливаем 0 для Application.Handle
  Application.Handle := 0;
  //т.к. FBarCode создается в методе Init, а метод Init может не
  //заработать, если версии приложения и плагина не равны
  if Assigned(FBarCode) then
  begin
    FBarCode.Stop;
    FreeAndNil(FBarCode);
  end;
  FreeAndNil(FIni);
  if Assigned(FAppProvider) then
    FreeAndNil(FAppProvider);
  inherited;
end;

procedure TPlugin.ShowException(const AMessage: string);
begin
  Application.MessageBox(
    PChar(AMessage),
    PChar('Ошибка плагина'),
    MB_ICONERROR or MB_OK or MB_TOPMOST);
end;

function TPlugin.GetLastError: WideString;
begin
  Result := FLastError;
end;

function TPlugin.GetVersion: Integer;
begin
  Result := PLUGIN_VERSION;
end;

procedure TPlugin.Init(AppProvider: IPluginAppProvider);
begin
  try
    Application.Title := AppProvider.GetAppTitle;
    FAppProvider := TAppProvider.Create(AppProvider);

    //Считаем настройки из Ini
    FComPort := FIni.GetCOMValue;

    //Запустим сканер штрих кодов
    FBarCode := TBarCode.Create(FComPort, AppProvider.GetAppCache.GetDBName);
    FBarCode.Start;
  except
    on E: Exception do
    begin
      ShowException(E.Message);
    end;
  end;
end;

procedure TPlugin.InitMainMenu(AMenu: IPluginMainMenu);
begin
  try
    AMenu.AddCommand('Настройки сканера штрих-кодов', 0);
  except
    on E: Exception do
    begin
      ShowException(E.Message);
    end;
  end;
end;

procedure TPlugin.CallMainMenu(AIndex: Integer);
begin
  try
    // получаем AppHandle для ассоциирования плагина с главным приложеним
    // Внимание! В деструкторе обязательно присваиваем 0 для Application.Handle
    Application.Handle := FAppProvider.GetAppHandle;
    case AIndex of
      0: ShowForm;
    end
  except
    on E: Exception do
    begin
      ShowException(E.Message);
    end;
  end;
end;

procedure TPlugin.ShowForm;
var
  frm: TfrmProperties;
begin
  frm := TfrmProperties.Create(FIni);
  try
    if frm.ShowModal = mrOK then
      //Перенастроим порт
      FBarCode.Restart(frm.ComPort);
  finally
    FreeAndNil(frm);
  end;
end;

end.
