unit Plugin;

interface

uses
  Windows,
  SysUtils,

  PluginIntf,
  PluginIntfWrapper;

type
  TPlugin = class(TInterfacedObject, IPlugin, IPluginEvents)
  private
    FLastError: WideString;
    FAppProvider: TAppProvider;
    procedure ShowException(const AMessage: string);
  public
    //   IPlugin
    function GetVersion(): Integer; safecall;
    function GetLastError(): WideString; safecall;
    procedure Init(AppProvider: IPluginAppProvider); safecall;
    //   IPluginEvents
    function AfterCreateDocument(ADocument: IPluginPDMVersion; AFile: WideString
      ): TCallResult; safecall;
    function AfterEditObject(AParent, AObject: IPluginPDMVersion; AIdLink: Integer;
      ALinkType, ANewProduct, ANewState: WideString): TCallResult; safecall;
    function CheckAttachment(Version: IPluginPDMVersion; out RejectReason: WideString): TCallResult; safecall;
    function BeforeUnloadingPlugin: TCallResult; safecall;
    function BeforCreateTask(AVersions: IPluginPDMVersions; out PluginTask: Boolean): TCallResult; safecall;
  public
    constructor Create;
    destructor Destroy; override;
  end;


function LoadPlugin(): IPlugin; safecall;


implementation

uses
  Forms,

  PluginUtils;
  

function LoadPlugin(): IPlugin;
begin
  Result := TPlugin.Create;
end;


{ TPlugin }

constructor TPlugin.Create;
begin

end;

destructor TPlugin.Destroy;
begin
  //   0  Application.Handle
  Application.Handle := 0;
  if Assigned(FAppProvider) then
    FreeAndNil(FAppProvider);
  inherited;
end;

procedure TPlugin.ShowException(const AMessage: string);
begin
  Application.MessageBox(
    PChar(AMessage),
    PChar(' '),
    MB_ICONERROR or MB_OK or MB_TOPMOST);
end;

function TPlugin.GetLastError: WideString;
begin
  Result := FLastError;
end;

function TPlugin.GetVersion: Integer;
begin
  Result := PLUGIN_VERSION;
end;

procedure TPlugin.Init(AppProvider: IPluginAppProvider);
begin
  try
    FAppProvider := TAppProvider.Create(AppProvider);
  except
    on E: Exception do
    begin
      ShowException(E.Message);
    end;
  end;
end;

function TPlugin.AfterCreateDocument(ADocument: IPluginPDMVersion;
  AFile: WideString): TCallResult;
var
  stMessage: string;
  ds: IPluginDataSet;
begin
  try
    //    GetInfoAboutVersion   2
    ds := FAppProvider.AppMain.GetInfoAboutVersion(ADocument.IdCheckout, '', '', '', ADocument.IdVersion, 2);
    //     MessageBox
    stMessage := GetDsResult(ds);
    MessageBox(FAppProvider.GetAppHandle, PChar(stMessage), '  GetInfoAboutVersion inMode = 2', MB_OK);
    //     .  ,    
    Result := crOk;
  except
    on E: Exception do
    begin
      //       (. TPlugin.GetLastError)
      FLastError := E.Message;
      //     .   ,    .
      Result := crError;

      ShowException(E.Message);
    end;
  end;
end;

function TPlugin.AfterEditObject(AParent, AObject: IPluginPDMVersion;
  AIdLink: Integer; ALinkType, ANewProduct, ANewState: WideString): TCallResult;
begin

end;

function TPlugin.BeforCreateTask(AVersions: IPluginPDMVersions;
  out PluginTask: Boolean): TCallResult;
begin
  PluginTask := True;
  try
    MessageBox(FAppProvider.GetAppHandle, PChar(AVersions.Version(0).Product), '  GetInfoAboutVersion inMode = 2', MB_OK);
    //     .  ,    
    Result := crOk;
  except
    on E: Exception do
    begin
      //       (. TPlugin.GetLastError)
      FLastError := E.Message;
      //     .   ,    .
      Result := crError;

      ShowException(E.Message);
    end;
  end;
end;

function TPlugin.BeforeUnloadingPlugin: TCallResult;
begin
  Result := crOk;
end;

function TPlugin.CheckAttachment(Version: IPluginPDMVersion;
  out RejectReason: WideString): TCallResult;
begin
  Result := crOk;
end;

end.
