unit Plugin;

interface

uses
  Windows,
  SysUtils,

  PluginIntf,
  PluginIntfWrapper;

type
  TPlugin = class(TInterfacedObject, IPlugin, IPluginCoolCommands)
  private
    FLastError: WideString;
    FAppProvider: TAppProvider;
    procedure ShowForm;
    procedure ShowException(const AMessage: string);
  private
    // IPlugin
    function GetVersion(): Integer; safecall;
    function GetLastError(): WideString; safecall;
    procedure Init(AppProvider: IPluginAppProvider); safecall;
    // IPluginCoolCommands
    procedure InitCoolCommands(ACCPanel: IPluginCoolCommandsPanel); safecall;
    procedure CallCoolCommand(AIndex: Integer); safecall;
  public
    constructor Create;
    destructor Destroy; override;
  end;


function LoadPlugin(): IPlugin; safecall;


implementation

uses
  Dialogs,
  Forms,
  Form1;


function LoadPlugin(): IPlugin;
begin
  Result := TPlugin.Create;
end;


{ TPlugin }

constructor TPlugin.Create;
begin
end;

destructor TPlugin.Destroy;
begin
  //   0  Application.Handle
  Application.Handle := 0;
  if Assigned(FAppProvider) then
    FreeAndNil(FAppProvider);
  inherited;
end;

procedure TPlugin.ShowException(const AMessage: string);
begin
  Application.MessageBox(
    PChar(AMessage),
    PChar(' '),
    MB_ICONERROR or MB_OK or MB_TOPMOST);
end;

function TPlugin.GetLastError: WideString;
begin
  Result := FLastError;
end;

function TPlugin.GetVersion: Integer;
begin
  Result := PLUGIN_VERSION;
end;

procedure TPlugin.Init(AppProvider: IPluginAppProvider);
begin
  try
    FAppProvider := TAppProvider.Create(AppProvider);
  except
    on E: Exception do
    begin
      ShowException(E.Message);
    end;
  end;
end;

procedure TPlugin.InitCoolCommands(ACCPanel: IPluginCoolCommandsPanel);
var
  SelectedObjects: IPluginPDMVersions;
begin
  try
    SelectedObjects := FAppProvider.GetSelectedObjects;
    if SelectedObjects.Count = 1 then
      if SelectedObjects.Version(0).TypeObj = '' then
        ACCPanel.AddCommand('   ', ' 1', 'IMAGE_1', 0);
  except
    on E: Exception do
    begin
      ShowException(E.Message);
    end;
  end;
end;

procedure TPlugin.CallCoolCommand(AIndex: Integer);
begin
  try
    //  AppHandle      .
    // !     0  Application.Handle
    Application.Handle := FAppProvider.GetAppHandle;

    case AIndex of
      0: ShowForm;
    else
      FLastError := '  ';
    end;
  except
    on E: Exception do
    begin
      ShowException(E.Message);
    end;
  end;
end;

procedure TPlugin.ShowForm;
var
  frm: TfrmForm1;
begin
  frm := TfrmForm1.Create(nil, FAppProvider);
  try
    frm.ShowModal;
  finally
    FreeAndNil(frm);
  end;
end;

end.
