unit BarCodeCom;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes;

type
  TByteArray = array[0..$FF] of Byte;

type
  //    COM-
  TComThread = class(TThread)
  private
    FOnCompleteEvent: TNotifyEvent;
    FOverRead: TOverlapped;
    FComPortData: string;
    FComPortHandle: Thandle;
    function ArrayOfByteToStr(Arr: TByteArray): string;
  protected
    procedure Execute; override;
    procedure TerminateEx;
    procedure DoCompleteEvent;
  public
    constructor Create(AHCom: THandle); reintroduce;
    property OnDataArrived: TNotifyEvent read FOnCompleteEvent write FOnCompleteEvent;
    property ComPortData: string read FComPortData;
  end;

  TBarCode = class
  strict private
    FComPortHandle: THandle;
    FComPort: string;
    FComThread: TComThread;
    FDBName: string;
    procedure InitPort;
    function OpenPort: Boolean;
    procedure ClosePort;
    function GetURLFromBarCode(ABarCode: string): string;
    procedure DoBarCodeArrived(Sender: TObject);
  public
    constructor Create(AComPort, ADBName: string);
    destructor Destroy; override;
    procedure Start;
    procedure Stop;
    procedure Restart(AComPort: string);
  end;

implementation

uses
  Forms,
  ShellApi,
  RegularExpressions;

{ TBarCode }

constructor TBarCode.Create(AComPort, ADBName: string);
begin
  FComPort := AComPort;
  FDBName := ADBName;
end;

destructor TBarCode.Destroy;
begin
  //
  inherited;
end;

procedure TBarCode.DoBarCodeArrived(Sender: TObject);
var
  URL: string;
begin
  //    COM-
  URL := GetURLFromBarCode(FComThread.FComPortData);
  if URL <> '' then
    //.  dll    ,  -
    //:()       
    //       -
    ShellExecute(0, nil, PChar(Application.ExeName), PChar(URL), nil, SW_SHOWNORMAL);
end;

function TBarCode.OpenPort: Boolean;
begin
  Result := False;

  //  COM-
  FComPortHandle := CreateFile(PChar(FComPort),                  // 
                               GENERIC_READ or GENERIC_WRITE,    //     
                               0,                                //    ,    
                               nil,                              // ,     nil
                               OPEN_EXISTING,                    // ,   OPEN_EXISTING
                               FILE_FLAG_OVERLAPPED,
                               0
                              );

  if FComPortHandle = INVALID_HANDLE_VALUE then
    Exit;

  // 
  InitPort;

  Result := True;
end;

procedure TBarCode.ClosePort;
begin
  // 
  if FComPortHandle <> INVALID_HANDLE_VALUE then
    CloseHandle(FComPortHandle);
end;

procedure TBarCode.InitPort;
var
  DCB: TDCB; //,   
begin
  //      ,   ,
  //   
  GetCommState(FComPortHandle, DCB);

  DCB.BaudRate := 9600;       // 
  DCB.Parity := NoParity;     //  
  DCB.ByteSize := 8;          //   
  DCB.StopBits := ONESTOPBIT; //  
  DCB.EvtChar := #13;         //        #13

  //  ,   
  SetCommState(FComPortHandle, Dcb);

  //     /  
  //     /
  PurgeComm(FComPortHandle, PURGE_TXCLEAR or PURGE_RXCLEAR);

  //     DCB.EvtChar := #13;
  SetCommMask(FComPortHandle, EV_RXFLAG);
end;

procedure TBarCode.Restart(AComPort: string);
begin
  Stop;
  FComPort := AComPort;
  Start;
end;

procedure TBarCode.Start;
begin
  if OpenPort then
  begin
    //      
    FComThread := TComThread.Create(FComPortHandle);
    FComThread.OnDataArrived := DoBarCodeArrived;
    FComThread.FreeOnTerminate := False;
    OutputDebugString( PChar('  ' + IntToStr(FComThread.ThreadID)));
  end;
  //else
    //  
    //     
end;

procedure TBarCode.Stop;
begin
  // 
  if Assigned(FComThread) then
  begin
    FComThread.TerminateEx;
    //FComThread.WaitFor;
    FComThread.Free;
  end;

  // 
  ClosePort;
end;

function TBarCode.GetURLFromBarCode(ABarCode: string): string;
{
        P12345-1 
      P - ,      
      12345 - id 
      1 -    
}
var
  Mass: TArray<string>;
  RegExpStr: string;
  ParamString: string;
begin
  Result := '';

  ParamString := '';
  if Application.Title = ':' then
  begin
    RegExpStr := '[P]+[0-9]+-[0-9]';
    ParamString := 'pgsp://';
  end;
  if Application.Title = ':' then
  begin
    RegExpStr := '[O]+[0-9]+-[0-9]';
    ParamString := 'pord://';
  end;

  //    -  
  ABarCode := StringReplace(ABarCode, #$D, '', [rfReplaceAll]);
  if TRegEx.IsMatch(ABarCode, RegExpStr) then
  begin
    Mass := TRegEx.Split(ABarCode, '-',[roSingleLine]);
    //     P12345
    Mass[0] := Copy(Mass[0], 2, Length(Mass[0]));
    //    pgsp://PGS_TEST/12345/1
    Result := ParamString + FDBname + '/' + Mass[0] + '/' + Mass[1];
  end;
end;

{ TComThread }

constructor TComThread.Create(AHCom: THandle);
begin
  FComPortHandle := AHCom;
  FillChar(FOverRead, SizeOf(FOverRead), 0);
  FOverRead.hEvent := CreateEvent(nil, False, False, nil);
  //if FOverRead.hEvent = null then
  //  raise Exception.Create('    ');
  inherited Create;
end;

procedure TComThread.DoCompleteEvent;
begin
  if Assigned(FOnCompleteEvent) then
    FOnCompleteEvent(Self);
end;

procedure TComThread.Execute;
var
  ComStat: TComStat;
  dwMask, dwError: DWORD;
  Buf: TByteArray;
  dwRead: DWORD;
begin
  while not Terminated do
  begin
    if not WaitCommEvent(FComPortHandle, dwMask, @FOverRead) then
      if GetLastError = ERROR_IO_PENDING then
        WaitForSingleObject(FOverRead.hEvent, INFINITE);

    if (dwMask and EV_RXFLAG) = EV_RXFLAG then
    begin
      //   
      ClearCommError(FComPortHandle, dwError, @ComStat);
      dwRead := ComStat.cbInQue;
      FillChar(Buf, SizeOf(Buf), 0);
      if dwRead > 0 then
      begin
        ReadFile(FComPortHandle, Buf, Sizeof(Buf), dwRead, @FOverRead);
          //raise Exception.Create(IntToStr(GetLastError) + '    ');
        //  Buf   
        //   
        FComPortData := ArrayOfByteToStr(Buf);
        OutputDebugString(PChar('Data Arrived'));
        //,   
        DoCompleteEvent;
        dwMask := 0;
      end;
    end;

    Sleep(0);
  end;
end;

procedure TComThread.TerminateEx;
begin
  //   WaitForSingleObject
  //  
  SetEvent(FOverRead.hEvent);
  inherited;
end;

function TComThread.ArrayOfByteToStr(Arr: TByteArray): string;
var
  i: Integer;
begin
  Result := '';
  for i:= 0 to High(Arr) do
    Result := Result + Chr(Arr[i]);
  // 
  Result := Trim(Result);
end;

end.

